/**
 * 企业相关逻辑
 * 作者: 李欣鸣
 * 包括新注册企业，迁入企业
 * 
 */

import { MoveInEnterpriseParamConfig, RegisterEnterpriseParamConfig } from "../../config/checkParamConfig";
import { ERRORENUM } from "../../config/errorEnum";
import { EnterpriseInfoConfig } from "../../config/ojbectResultKeyConfig";
import { createEnterprise, deleteEnterprise,findEnterpriseCount, findEnterpriseByUscc, findEnterpriseList, groupFindEnterprise } from "../../data/enterprise/enterprise";
import { EnterpriseListConfig } from "../../config/ojbectResultKeyConfig";
import { BizError } from "../../util/bizError";
import { checkChange, checkParamater, extractData, getTaskId } from "../../util/tools"


/**
 * 新注册企业
 * @param uscc 孵化器统一信用代码
 * @param param 表单参数
 * @returns {isSuccess:true/false}
 */
export async function registerEnterprise(uscc:string, param) {
    const TaskId = getTaskId(uscc);
    checkParamater("新注册企业", RegisterEnterpriseParamConfig, param);
    let dataBaseInfo = await findEnterpriseByUscc(param.uscc);
    
    if ( dataBaseInfo&& dataBaseInfo.uscc)  throw new BizError(ERRORENUM.该企业已存在, `${param.uscc}已经在库中存在`);

    param.isCreate = true;
    await createEnterprise(uscc, TaskId, param);

    return {isSuccess:true};
}


/**
 * 新注册企业表单修改
 * @param uscc 孵化器统一信用代码
 * @param param 表单参数
 * @returns {isSuccess:true/false}
 */
export async function updateRegisterEnterprise(uscc:string, param) {
    checkParamater("新注册企业表单修改", RegisterEnterpriseParamConfig, param);
    let dataBaseInfo = await findEnterpriseByUscc(param.uscc);
    
    if (!dataBaseInfo || !dataBaseInfo.uscc) throw new BizError(ERRORENUM.未找到数据, `库中不存在${param.uscc}这个企业`);

    let changeList = checkChange(param, dataBaseInfo);
    if ( !changeList.length ) throw new BizError(ERRORENUM.数据无更新, `${param.uscc}数据无更新`);

    changeList.forEach(key => {
        dataBaseInfo[key] = param[key];
    });

    await dataBaseInfo.save();

    return {isSuccess:true};
}


/**
 * 迁入企业登记
 * @param uscc 孵化器统一信用代码
 * @param param 表单参数
 * @returns {isSuccess:true/false}
 */
export async function moveInEnterprise(uscc:string, param) {
    const TaskId = getTaskId(uscc);
    checkParamater("迁入企业登记", MoveInEnterpriseParamConfig, param);
    let dataBaseInfo = await findEnterpriseByUscc(param.uscc);
    
    if ( dataBaseInfo&& dataBaseInfo.uscc)  throw new BizError(ERRORENUM.该企业已存在, `${param.uscc}已经在库中存在`);

    await createEnterprise(uscc, TaskId, param);

    return {isSuccess:true};
}


/**
 * 修改迁入企业登记信息
 * @param uscc 孵化器统一信用代码
 * @param param 表单参数
 * @returns {isSuccess:true/false}
 */
export async function updateMoveInEnterprise(uscc:string, param) {
    checkParamater("修改迁入企业登记信息", MoveInEnterpriseParamConfig, param);
    let dataBaseInfo = await findEnterpriseByUscc(param.uscc);
    
    if (!dataBaseInfo || !dataBaseInfo.uscc) throw new BizError(ERRORENUM.未找到数据, `库中不存在${param.uscc}这个企业`);


    let changeList = checkChange(param, dataBaseInfo);
    if ( !changeList.length ) throw new BizError(ERRORENUM.数据无更新, `${param.uscc}数据无更新`);

    changeList.forEach(key => {
        dataBaseInfo[key] = param[key];
    });

    await dataBaseInfo.save();

    return {isSuccess:true};
    
}


/**
 * 根据企业id查询企业
 * @param uscc 企业统一信用代码
 */
export async function getEnterpriseByUscc(uscc:string) {
    let dataBaseInfo = await findEnterpriseByUscc(uscc);
    if (!dataBaseInfo || !dataBaseInfo.uscc) throw new BizError(ERRORENUM.未找到数据, `库中不存在${uscc}这个企业`);

    let data = extractData(EnterpriseInfoConfig, dataBaseInfo);

    return {data};
}


/**
 * 根据企业id删除草稿企业
 * @param uscc 企业统一信用代码
 */
export async function deleteEnterpriseByUscc(uscc:string) {
    let dataBaseInfo = await findEnterpriseByUscc(uscc);
    if (!dataBaseInfo || !dataBaseInfo.uscc) throw new BizError(ERRORENUM.未找到数据, `库中不存在${uscc}这个企业`);

    if (dataBaseInfo.draftLock) throw new BizError(ERRORENUM.已入库的企业不能删除, uscc);

    await deleteEnterprise(uscc);

    return {isSuccess:true};
}


/**
 * 获取在孵企业信息列表
 * @param time 新注册时间/迁入时间
 * @param fuHuaQiUscc 所属孵化器 
 * @param industry 行业领域
 * @param isNaturalPersonHolding 自然人控股
 * @param page 页数
 * @returns 
 */
export async function enterpriseList(time:number, fuHuaQiUscc:string, industry:number, isNaturalPersonHolding:boolean, page:number) {
    let selectParam:any = {};
    if (time) selectParam.logonTime = time; selectParam.timeOfImmigration = time;
    if (fuHuaQiUscc) selectParam.fuHuaQiUscc = fuHuaQiUscc;
    if (industry) selectParam.industry = industry;
    if (isNaturalPersonHolding) selectParam.isNaturalPersonHolding = isNaturalPersonHolding;

    let enterpriseMap = await groupFindEnterprise();
    let dataBaseList = await findEnterpriseList(selectParam, (page-1)*10 );
    let count = await findEnterpriseCount(selectParam);
    let dataList = [];
    dataBaseList.forEach( info => {
        let changeData = extractData(EnterpriseListConfig, info);
        const Count = enterpriseMap[info.uscc] || 0;
        dataList.push(changeData);
    });

    return {count, dataList};
}