/**
 * 企业融资相关逻辑
 * 作者: 李欣鸣
 * 包括新融资信息的增删改查
 * 
 */

import { FinancingParamConfig } from "../../config/checkParamConfig";
import { ERRORENUM } from "../../config/errorEnum";
import { EnterpriseFinancingInfoConfig } from "../../config/ojbectResultKeyConfig";
import { createFinancing, deleteEnterpriseFinancing, findAddByName, findEnterpriseFinancingByUscc, findFinancingCount, findFinancingInfoByTaskIdAndSucc, findFinancingList, groupFindFinancing } from "../../data/enterprise/financing";
import { FinancingListConfig } from "../../config/ojbectResultKeyConfig";
import { BizError } from "../../util/bizError";
import { checkChange, checkParamater, extractData, getTaskId } from "../../util/tools";
import { findEnterpriseInfoByName, findEnterpriseListByFuHuaQiUsccName } from "../../data/enterprise/enterprise";
import { findFuHuaQiByName } from "../../data/fuHuaQi/fuhuaqi";


/**
 * 创建企业融资信息
 * @param uscc 孵化器统一信用代码
 * @param param 表单配置
 * @returns {isSuccess:true/false}
 */
export async function createFinancingInfo(uscc, param) {
    const TaskId = getTaskId(uscc);

    checkParamater("录入企业融资信息", FinancingParamConfig, param);


    let enterpriseInfo = await findEnterpriseInfoByName(param.name);
    if (!enterpriseInfo) throw new BizError(ERRORENUM.该企业不存在, uscc, param.name);
    param.uscc = enterpriseInfo.uscc;

    let dataBaseInfo = await findFinancingInfoByTaskIdAndSucc(TaskId, param.uscc);
    
    if ( dataBaseInfo&& dataBaseInfo.uscc)  throw new BizError(ERRORENUM.该企业当月数据已存在, `${param.uscc}的融资信息已经存在`);

    await createFinancing(uscc, TaskId, param);

    return {isSuccess:true};
}

/**
 * 更新融资企业信息
 * @param uscc 孵化器统一信用代码
 * @param param 表单配置
 * @returns {isSuccess:true/false}
 */
export async function updateFinancingInfo(uscc, param) {
    const TaskId = getTaskId(uscc);

    checkParamater("更新融资企业信息", FinancingParamConfig, param);
    let dataBaseInfo = await findFinancingInfoByTaskIdAndSucc(TaskId, param.uscc);
    let changeList = checkChange(param, dataBaseInfo);

    if (!dataBaseInfo || !dataBaseInfo.uscc) throw new BizError(ERRORENUM.未找到数据, `库中不存在${TaskId}的任务`);
    if ( !changeList.length ) throw new BizError(ERRORENUM.数据无更新, `${param.uscc}数据无更新`);

    changeList.forEach(key => {
        dataBaseInfo[key] = param[key];
    });

    await dataBaseInfo.save();

    return {isSuccess:true};
}


/**
 * 根据企业id查询企业
 * @param uscc 企业统一信用代码
 */
export async function getEnterpriseFinancingByUscc(fuHuaQiUscc:string, uscc:string) {
    const TaskId = getTaskId(fuHuaQiUscc);
    let dataBaseInfo = await findEnterpriseFinancingByUscc(uscc, TaskId);
    if (!dataBaseInfo || !dataBaseInfo.uscc) throw new BizError(ERRORENUM.未找到数据, `库中不存在${uscc}这个企业`);

    let data = extractData(EnterpriseFinancingInfoConfig, dataBaseInfo);

    return {data};
}


/**
 * 根据企业id删除草稿企业
 * @param uscc 企业统一信用代码
 */
export async function deleteEnterpriseFinancingByUscc(fuHuaQiUscc:string, uscc:string) {
    const TaskId = getTaskId(fuHuaQiUscc);
    let dataBaseInfo = await findEnterpriseFinancingByUscc(uscc, TaskId);
    if (!dataBaseInfo || !dataBaseInfo.uscc) throw new BizError(ERRORENUM.未找到数据, `库中不存在${uscc}这个企业`);

    if (dataBaseInfo.draftLock) throw new BizError(ERRORENUM.已入库的企业不能删除, uscc);

    await deleteEnterpriseFinancing(uscc, TaskId);

    return {isSuccess:true};
}


/**
 * 融资企业名称模糊查询
 *
 */
export async function getFinancingByName(uscc:string) {
    let financingInfo = await findEnterpriseListByFuHuaQiUsccName(uscc);
    let dataList = [];
    financingInfo.forEach( info => {
        dataList.push({
            name:info.name,
            logonAdd:info.logonAdd, 
            operatingAdd:info.operatingAdd
        });
    });
    return dataList;
}


/**
 * 融资企业信息列表
 * @param monthData 数据月份 目前数据库没有该字段
 * @param fuHuaQiName 所属孵化器 
 * @param industry 行业领域
 * @param fuHuaQiInvestment 孵化器是否参与投资
 * @param page 页数 
 * @returns 
 */
export async function financingList(monthData:string, fuHuaQiName:string, industry:[Number], fuHuaQiInvestment:boolean, page:number) {
    let fuHuaQiInfo = await findFuHuaQiByName(fuHuaQiName);

    let selectParam:any = {};
    let year = monthData.substring(0,5);
    let month = monthData.slice(5, monthData.length);
    if (monthData) selectParam.year = year;  selectParam.month = month;
    if (fuHuaQiName) selectParam.fuHuaQiUscc = fuHuaQiInfo.uscc;
    if (industry) selectParam.industry = industry;
    if (fuHuaQiInvestment) selectParam.fuHuaQiInvestment = fuHuaQiInvestment;

    let enterpriseMap = await groupFindFinancing();
    let dataBaseList = await findFinancingList(selectParam, (page-1)*10 );
    let count = await findFinancingCount(selectParam);
    let dataList = [];
    dataBaseList.forEach( info => {
        let changeData = extractData(FinancingListConfig, info);
        const Count = enterpriseMap[info.uscc] || 0;
        dataList.push(changeData);
    });

    return {count, dataList};
}